<?php

namespace app\admin\controller;

use app\common\model\ai\Config;

class Aiconfig extends Botbase
{
    /**
     * @var Config
     */
    protected $model;
    /**
     * @var array
     */
    private $tabList;
    /**
     * @var array
     */
    private $aiDrivers;

    public function initialize()
    {
        parent::initialize(); // TODO: Change the autogenerated stub
        $this->model = new Config();
        $this->aiDrivers = [
            'weixin' => '微信AI',
            'qyk' => '青云客',
            'gpt' => 'ChatGpt'
        ];
        $this->tabList = [
            'basic' => ['title' => '基础设置', 'href' => url('index', ['name' => 'basic'])],
            'driver' => ['title' => '驱动设置', 'href' => url('index', ['name' => 'driver'])]
        ];
    }

    /**
     * 设置
     * @return mixed
     * @throws \think\Exception
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function index(){
        $name = input('name', 'basic');
        $tip = "<ul><li>微信OpenAi: <a target='_blank' href='https://openai.weixin.qq.com/'>https://openai.weixin.qq.com/</a></li>
<li>青云客Ai: <a target='_blank' href='http://api.qingyunke.com/'>http://api.qingyunke.com/</a></li>
</ul>";
        $settings = $this->model->getConf(['bot_id' => $this->bot['id']]);
        if(!empty($settings['wxids'])){
            $settings['wxids'] = explode(',', $settings['wxids']);
        }
        if(empty($settings)){
            $settings = [
                'driver' => 'weixin',
                'switch' => 1,
                'need_at' => 1,
                'show_question' => 1
            ];
        }

        $builder = new FormBuilder();
        $builder->setPostUrl(url('savePost'))
            ->setTabNav($this->tabList, $name)
            ->addFormItem('id', 'hidden', 'id', 'id');
            switch ($name){
                case 'driver':
                    $builder->setTip($tip)
                        ->addFormItem('weixin_legend', 'legend', '微信智能对话','微信智能对话' )
                        ->addFormItem('wx_appid', 'text', 'Appid', '微信智能对话的Appid')
                        ->addFormItem('wx_token', 'text', 'Token', '微信智能对话的Token')
                        ->addFormItem('wx_encoding_aes_key', 'text', 'EncodingAesKey', '微信智能对话的EncodingAesKey')
                        ->addFormItem('qyk_legend', 'legend', '青云客','青云客' )
                        ->addFormItem('qyk_appid', 'text', 'Appid', '填0就好');
                        /*->addFormItem('gpt_legend', 'legend', 'chatGpt','chatGpt' )
                        ->addFormItem('gpt_appid', 'text', 'OpenaiId', 'OpenaiId的OpenaiId')*/
                    break;
                default:
                    $builder->addFormItem('switch', 'radio', '开启', '是否开启', [1=>'是', 0 => '否'], 'required')
                        ->addFormItem('need_at', 'radio', '被@后触发', '群聊中是否需要被at才回答', [1=>'是', 0 => '否'])
                        ->addFormItem('show_question', 'radio', '答案是否带上问题', '答案是否带上问题', [1=>'是', 0 => '否'])
                        ->addFormItem('driver', 'radio', 'AI驱动', 'AI驱动', $this->aiDrivers)
                        ->addFormItem('wxids', 'chosen_multi', '作用对象', '指定的作用对象才会生效', $this->getMembers(), 'required')
                        ->addFormItem('time_on', 'time', '上班时间', '最早的时间', [], 'required')
                        ->addFormItem('time_off', 'time', '下班时间', '晚上最迟', [], 'required');
                    break;
            }

        return $builder->setFormData($settings)->show();
    }

    public function savePost($jump_to = '/undefined', $data = [])
    {
        $jump_to = url('index');
        $post_data = input('post.');
        unset($post_data['__token__']);
        foreach ($post_data as $k => $v){
            if($config = $this->model->getOneByMap(['bot_id' => $this->bot['id'], 'key' => $k],  true, true)){
                $this->model->updateOne(['id' => $config['id'], 'value' => $v]);
            }else{
                $this->model->addOne([
                    'admin_id' => $this->adminInfo['id'],
                    'bot_id' => $this->bot['id'],
                    'key' => $k,
                    'value'=> $v
                ]);
            }
        }
        $this->model->getConf(['bot_id' => $this->bot['id']], '', true);
        $this->success('数据保存成功', $jump_to);
    }
}